/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package lucene;


import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Paths;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.FSDirectory;

/** Simple command-line based search demo. */
public class SearchFiles {

  private SearchFiles() {}

  /** Simple command-line based search demo. */
  public static void main(String[] args) throws Exception {
	  String index = "index";
	  String field = "contents";
	    
	    // regular search
//	    String queryString = "computer";
	    
	    //wildcard query
//	    String queryString = "te*t";
	    
	    //fuzzy query
//	    String queryString = "roam~2";
	    
	   //phrase query 
//	    String queryString = "\"apache lucene\"~5";

	    //boolean search
//	    String queryString = "\"networks\" AND \"protocol\"";


	  //boosted search
	  String queryString = "computer^10 crime";
	    
	  int hitsPerPage = 10;
	    
	  IndexReader reader = DirectoryReader.open(FSDirectory.open(Paths.get(index)));
	  IndexSearcher searcher = new IndexSearcher(reader);
	  Analyzer analyzer = new StandardAnalyzer();

	  BufferedReader in = null;
	  QueryParser parser = new QueryParser(field, analyzer);
  
	  Query query = parser.parse(queryString);

	  System.out.println("Searching for: " + query.toString(field));
		  
	  doPagingSearch(in, searcher, query, hitsPerPage);    
		  
	  reader.close();
  }

  public static void doPagingSearch(BufferedReader in, IndexSearcher searcher, Query query, 
                                     int hitsPerPage) throws IOException {
 
    TopDocs results = searcher.search(query, hitsPerPage);
    ScoreDoc[] hits = results.scoreDocs;
    
    int numTotalHits = results.totalHits;
    System.out.println(numTotalHits + " total matching documents");

    int start = 0;
    int end = Math.min(numTotalHits, hitsPerPage);
        
	if (end > hits.length) 
	{
	    System.out.println("Only results 1 - " + hits.length +" of " + numTotalHits + " total matching documents collected.");
	}
      
    for (int i = start; i < end; i++) {
        

    Document doc = searcher.doc(hits[i].doc);
    String path = doc.get("path");
    
    if (path != null) 
    {
          System.out.println((i+1) + ". " + path);
          String title = doc.get("title");
          
          if (title != null) 
          {
              System.out.println("   Title: " + doc.get("title"));
          }
    } 
    else 
    {
          System.out.println((i+1) + ". " + "No path for this document");
    }
                  
  }

}
  
}
